@@ -458,32 +458,78 @@ egg_markdown_add_pending_header (EggMarkdown *self, const gchar *line)
 	return ret;
 }
 
+/**
+ * egg_markdown_count_chars_in_word:
+ **/
+static guint
+egg_markdown_count_chars_in_word (const gchar *text, gchar find)
+{
+	guint i;
+	guint len;
+	guint count = 0;
+
+	/* get length */
+	len = egg_strlen (text, EGG_MARKDOWN_MAX_LINE_LENGTH);
+	if (len == 0)
+		goto out;
+
+	/* find matching chars */
+	for (i=0; i<len; i++) {
+		if (text[i] == find)
+			count++;
+	}
+out:
+	return count;
+}
+
 /**
  * egg_markdown_word_is_code:
  **/
 static gboolean
 egg_markdown_word_is_code (const gchar *text)
 {
+	/* already code */
 	if (g_str_has_prefix (text, "`"))
 		return FALSE;
 	if (g_str_has_suffix (text, "`"))
 		return FALSE;
+
+	/* paths */
 	if (g_str_has_prefix (text, "/"))
 		return TRUE;
+
+	/* bugzillas */
 	if (g_str_has_prefix (text, "#"))
 		return TRUE;
+
+	/* uri's */
 	if (g_str_has_prefix (text, "http://"))
 		return TRUE;
 	if (g_str_has_prefix (text, "https://"))
 		return TRUE;
 	if (g_str_has_prefix (text, "ftp://"))
 		return TRUE;
+
+	/* patch files */
 	if (g_strrstr (text, ".patch") != NULL)
 		return TRUE;
+	if (g_strrstr (text, ".diff") != NULL)
+		return TRUE;
+
+	/* function names */
 	if (g_strrstr (text, "()") != NULL)
 		return TRUE;
+
+	/* email addresses */
 	if (g_strrstr (text, "@") != NULL)
 		return TRUE;
+
+	/* compiler defines */
+	if (text[0] != '_' &&
+	    egg_markdown_count_chars_in_word (text, '_') > 1)
+		return TRUE;
+
+	/* nothing special */
 	return FALSE;
 }
 
@@ -498,7 +544,7 @@ egg_markdown_word_auto_format_code (const gchar *text)
 	gchar **words;
 	gboolean ret = FALSE;
 
-	/* split sentance up with space */
+	/* split sentence up with space */
 	words = g_strsplit (text, " ", -1);
 
 	/* search each word */
@@ -1099,6 +1145,7 @@ egg_markdown_test (EggTest *test)
 	markdown = "this is http://www.hughsie.com/with_spaces_in_url inline link\n";
 	markdown_expected = "this is <tt>http://www.hughsie.com/with_spaces_in_url</tt> inline link";
 	egg_test_title (test, "markdown (autocode)");
+	egg_markdown_set_autocode (self, TRUE);
 	text = egg_markdown_parse (self, markdown);
 	if (egg_strequal (text, markdown_expected))
 		egg_test_success (test, NULL);
@@ -1250,6 +1297,15 @@ egg_markdown_test (EggTest *test)
 		egg_test_failed (test, "failed, got '%s'", text);
 	g_free (text);
 
+	/************************************************************/
+	egg_test_title (test, "markdown (autotext underscore)");
+	text = egg_markdown_parse (self, "This isn't CONFIG_UEVENT_HELPER_PATH present");
+	if (egg_strequal (text, "This isn't <tt>CONFIG_UEVENT_HELPER_PATH</tt> present"))
+		egg_test_success (test, NULL);
+	else
+		egg_test_failed (test, "failed, got '%s'", text);
+	g_free (text);
+
 	/************************************************************/
 	markdown = "*Thu Mar 12 12:00:00 2009* Dan Walsh <dwalsh@redhat.com> - 2.0.79-1\n"
 		   "- Update to upstream \n"
@@ -1260,11 +1316,11 @@ egg_markdown_test (EggTest *test)
 		   "- Add patch from ajax to accellerate X SELinux \n"
 		   "- Update eparis patch\n";
 	markdown_expected =
-		   "<i>Thu Mar 12 12:00:00 2009</i> Dan Walsh &lt;dwalsh@redhat.com&gt; - 2.0.79-1\n"
+		   "<i>Thu Mar 12 12:00:00 2009</i> Dan Walsh <tt>&lt;dwalsh@redhat.com&gt;</tt> - 2.0.79-1\n"
 		   "• Update to upstream\n"
 		   "• Netlink socket handoff patch from Adam Jackson.\n"
 		   "• AVC caching of compute_create results by Eric Paris.\n"
-		   "<i>Tue Mar 10 12:00:00 2009</i> Dan Walsh &lt;dwalsh@redhat.com&gt; - 2.0.78-5\n"
+		   "<i>Tue Mar 10 12:00:00 2009</i> Dan Walsh <tt>&lt;dwalsh@redhat.com&gt;</tt> - 2.0.78-5\n"
 		   "• Add patch from ajax to accellerate X SELinux\n"
 		   "• Update eparis patch";
 	egg_test_title (test, "markdown (end of bullett)");
