@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
  *
- * Copyright (C) 2007 Richard Hughes <richard@hughsie.com>
+ * Copyright (C) 2007-2008 Richard Hughes <richard@hughsie.com>
  *
  * Licensed under the GNU General Public License Version 2
  *
@@ -33,31 +33,23 @@
 #include <unistd.h>
 #endif /* HAVE_UNISTD_H */
 #include <glib/gi18n.h>
-#include <dbus/dbus-glib.h>
-#include <libgbus.h>
 #include <gconf/gconf-client.h>
+#include <packagekit-glib/packagekit.h>
+#include <devkit-power-gobject/devicekit-power.h>
+
+#include "egg-debug.h"
+#include "egg-string.h"
 
-#include <pk-debug.h>
-#include <pk-control.h>
 #include "gpk-common.h"
+#include "gpk-session.h"
 #include "gpk-auto-refresh.h"
+#include "gpk-enum.h"
 
-static void     gpk_auto_refresh_class_init	(GpkAutoRefreshClass *klass);
-static void     gpk_auto_refresh_init		(GpkAutoRefresh      *arefresh);
 static void     gpk_auto_refresh_finalize	(GObject            *object);
 
 #define GPK_AUTO_REFRESH_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GPK_TYPE_AUTO_REFRESH, GpkAutoRefreshPrivate))
 #define GPK_AUTO_REFRESH_PERIODIC_CHECK		60*60	/* force check for updates every this much time */
-
-#define GS_DBUS_SERVICE				"org.gnome.ScreenSaver"
-#define GS_DBUS_PATH				"/org/gnome/ScreenSaver"
-#define GS_DBUS_INTERFACE			"org.gnome.ScreenSaver"
-
-#define GPM_DBUS_SERVICE			"org.freedesktop.PowerManagement"
-#define GPM_DBUS_PATH				"/org/freedesktop/PowerManagement"
-#define GPM_DBUS_PATH_INHIBIT			"/org/freedesktop/PowerManagement/Inhibit"
-#define GPM_DBUS_INTERFACE			"org.freedesktop.PowerManagement"
-#define GPM_DBUS_INTERFACE_INHIBIT		"org.freedesktop.PowerManagement.Inhibit"
+#define GPK_UPDATES_LOGIN_TIMEOUT		3	/* seconds */
 
 /*
  * at startup, after a small delay, force a GetUpdates call
@@ -71,20 +63,19 @@ struct GpkAutoRefreshPrivate
 	gboolean		 session_idle;
 	gboolean		 on_battery;
 	gboolean		 network_active;
-	gboolean		 session_delay;
-	gboolean		 sent_get_updates;
-	LibGBus			*gbus_gs;
-	LibGBus			*gbus_gpm;
+	gboolean		 force_get_updates_login;
+	guint			 force_get_updates_login_timeout_id;
+	guint			 timeout_id;
+	DkpClient		*client;
 	GConfClient		*gconf_client;
-	DBusGProxy		*proxy_gs;
-	DBusGProxy		*proxy_gpm;
-	DBusGConnection		*connection;
+	GpkSession		*session;
 	PkControl		*control;
 };
 
 enum {
 	REFRESH_CACHE,
 	GET_UPDATES,
+	GET_UPGRADES,
 	LAST_SIGNAL
 };
 
@@ -112,6 +103,11 @@ gpk_auto_refresh_class_init (GpkAutoRefreshClass *klass)
 			      G_TYPE_FROM_CLASS (object_class), G_SIGNAL_RUN_LAST,
 			      0, NULL, NULL, g_cclosure_marshal_VOID__VOID,
 			      G_TYPE_NONE, 0);
+	signals [GET_UPGRADES] =
+		g_signal_new ("get-upgrades",
+			      G_TYPE_FROM_CLASS (object_class), G_SIGNAL_RUN_LAST,
+			      0, NULL, NULL, g_cclosure_marshal_VOID__VOID,
+			      G_TYPE_NONE, 0);
 }
 
 /**
@@ -122,7 +118,7 @@ gpk_auto_refresh_signal_refresh_cache (GpkAutoRefresh *arefresh)
 {
 	g_return_val_if_fail (GPK_IS_AUTO_REFRESH (arefresh), FALSE);
 
-	pk_debug ("emitting refresh-cache");
+	egg_debug ("emitting refresh-cache");
 	g_signal_emit (arefresh, signals [REFRESH_CACHE], 0);
 	return TRUE;
 }
@@ -135,61 +131,43 @@ gpk_auto_refresh_signal_get_updates (GpkAutoRefresh *arefresh)
 {
 	g_return_val_if_fail (GPK_IS_AUTO_REFRESH (arefresh), FALSE);
 
-	pk_debug ("emitting get-updates");
+	egg_debug ("emitting get-updates");
 	g_signal_emit (arefresh, signals [GET_UPDATES], 0);
 	return TRUE;
 }
 
 /**
- * gpk_auto_refresh_convert_frequency:
- *
- * Return value: The number of seconds for the frequency period,
- * or zero for never or no schema
+ * gpk_auto_refresh_signal_get_upgrades:
  **/
-static guint
-gpk_auto_refresh_convert_frequency (PkFreqEnum freq)
+static gboolean
+gpk_auto_refresh_signal_get_upgrades (GpkAutoRefresh *arefresh)
 {
-	if (freq == PK_FREQ_ENUM_UNKNOWN) {
-		pk_warning ("no schema");
-		return 0;
-	}
-	if (freq == PK_FREQ_ENUM_NEVER) {
-		return 0;
-	}
-	if (freq == PK_FREQ_ENUM_HOURLY) {
-		return 60*60;
-	}
-	if (freq == PK_FREQ_ENUM_DAILY) {
-		return 60*60*24;
-	}
-	if (freq == PK_FREQ_ENUM_WEEKLY) {
-		return 60*60*24*7;
-	}
-	pk_warning ("unknown frequency enum");
-	return 0;
+	g_return_val_if_fail (GPK_IS_AUTO_REFRESH (arefresh), FALSE);
+
+	egg_debug ("emitting get-upgrades");
+	g_signal_emit (arefresh, signals [GET_UPGRADES], 0);
+	return TRUE;
 }
 
 /**
- * gpk_auto_refresh_convert_frequency_text:
+ * gpk_auto_refresh_get_frequency_prefs:
  **/
 static guint
-gpk_auto_refresh_convert_frequency_text (GpkAutoRefresh *arefresh, const gchar *key)
+gpk_auto_refresh_get_frequency_prefs (GpkAutoRefresh *arefresh, const gchar *key)
 {
-	const gchar *freq_text;
-	PkFreqEnum freq;
+	guint value;
 
 	g_return_val_if_fail (GPK_IS_AUTO_REFRESH (arefresh), 0);
 
 	/* get from gconf */
-	freq_text = gconf_client_get_string (arefresh->priv->gconf_client, key, NULL);
-	if (freq_text == NULL) {
-		pk_warning ("no schema for %s", key);
-		return 0;
+	value = gconf_client_get_int (arefresh->priv->gconf_client, key, NULL);
+	if (value == 0) {
+		egg_warning ("no schema for %s, using daily", key);
+		value = 86400;
 	}
 
 	/* convert to enum and get seconds */
-	freq = pk_freq_enum_from_text (freq_text);
-	return gpk_auto_refresh_convert_frequency (freq);
+	return value;
 }
 
 /**
@@ -198,42 +176,49 @@ gpk_auto_refresh_convert_frequency_text (GpkAutoRefresh *arefresh, const gchar *
 static gboolean
 gpk_auto_refresh_maybe_refresh_cache (GpkAutoRefresh *arefresh)
 {
-	guint time;
+	guint time_s;
 	guint thresh;
 	gboolean ret;
 
 	g_return_val_if_fail (GPK_IS_AUTO_REFRESH (arefresh), FALSE);
 
+	/* if we don't want to auto check for updates, don't do this either */
+	thresh = gpk_auto_refresh_get_frequency_prefs (arefresh, GPK_CONF_FREQUENCY_GET_UPDATES);
+	if (thresh == 0) {
+		egg_debug ("not when policy is to never refresh");
+		return FALSE;
+	}
+
 	/* not on battery */
 	if (arefresh->priv->on_battery) {
-		pk_debug ("not when on battery");
+		egg_debug ("not when on battery");
 		return FALSE;
 	}
 
 	/* only do the refresh cache when the user is idle */
-	if (arefresh->priv->session_idle == FALSE) {
-		pk_debug ("not when session active");
+	if (!arefresh->priv->session_idle) {
+		egg_debug ("not when session active");
 		return FALSE;
 	}
 
 	/* get this each time, as it may have changed behind out back */
-	thresh = gpk_auto_refresh_convert_frequency_text (arefresh, GPK_CONF_FREQUENCY_REFRESH_CACHE);
+	thresh = gpk_auto_refresh_get_frequency_prefs (arefresh, GPK_CONF_FREQUENCY_REFRESH_CACHE);
 	if (thresh == 0) {
-		pk_debug ("not when policy is to never refresh");
+		egg_debug ("not when policy is to never refresh");
 		return FALSE;
 	}
 
 	/* get the time since the last refresh */
 	ret = pk_control_get_time_since_action (arefresh->priv->control,
-						PK_ROLE_ENUM_REFRESH_CACHE, &time, NULL);
-	if (ret == FALSE) {
-		pk_warning ("failed to get last time");
+						PK_ROLE_ENUM_REFRESH_CACHE, &time_s, NULL);
+	if (!ret) {
+		egg_warning ("failed to get last time");
 		return FALSE;
 	}
 
 	/* have we passed the timout? */
-	if (time < thresh) {
-		pk_debug ("not before timeout, thresh=%u, now=%u", thresh, time);
+	if (time_s < thresh) {
+		egg_debug ("not before timeout, thresh=%u, now=%u", thresh, time_s);
 		return FALSE;
 	}
 
@@ -247,30 +232,39 @@ gpk_auto_refresh_maybe_refresh_cache (GpkAutoRefresh *arefresh)
 static gboolean
 gpk_auto_refresh_maybe_get_updates (GpkAutoRefresh *arefresh)
 {
-	guint time;
+	guint time_s;
 	guint thresh;
 	gboolean ret;
 
 	g_return_val_if_fail (GPK_IS_AUTO_REFRESH (arefresh), FALSE);
 
+	if (!arefresh->priv->force_get_updates_login) {
+		arefresh->priv->force_get_updates_login = TRUE;
+		if (gconf_client_get_bool (arefresh->priv->gconf_client, GPK_CONF_FORCE_GET_UPDATES_LOGIN, NULL)) {
+			egg_debug ("forcing get update due to GConf");
+			gpk_auto_refresh_signal_get_updates (arefresh);
+			return TRUE;
+		}
+	}
+
 	/* get this each time, as it may have changed behind out back */
-	thresh = gpk_auto_refresh_convert_frequency_text (arefresh, GPK_CONF_FREQUENCY_GET_UPDATES);
+	thresh = gpk_auto_refresh_get_frequency_prefs (arefresh, GPK_CONF_FREQUENCY_GET_UPDATES);
 	if (thresh == 0) {
-		pk_debug ("not when policy is to never refresh");
+		egg_debug ("not when policy is set to never get updates");
 		return FALSE;
 	}
 
 	/* get the time since the last refresh */
 	ret = pk_control_get_time_since_action (arefresh->priv->control,
-						PK_ROLE_ENUM_GET_UPDATES, &time, NULL);
-	if (ret == FALSE) {
-		pk_warning ("failed to get last time");
+						PK_ROLE_ENUM_GET_UPDATES, &time_s, NULL);
+	if (!ret) {
+		egg_warning ("failed to get last time");
 		return FALSE;
 	}
 
 	/* have we passed the timout? */
-	if (time < thresh) {
-		pk_debug ("not before timeout, thresh=%u, now=%u", thresh, time);
+	if (time_s < thresh) {
+		egg_debug ("not before timeout, thresh=%u, now=%u", thresh, time_s);
 		return FALSE;
 	}
 
@@ -278,73 +272,138 @@ gpk_auto_refresh_maybe_get_updates (GpkAutoRefresh *arefresh)
 	return TRUE;
 }
 
+/**
+ * gpk_auto_refresh_maybe_get_upgrades:
+ **/
+static gboolean
+gpk_auto_refresh_maybe_get_upgrades (GpkAutoRefresh *arefresh)
+{
+	guint time_s;
+	guint thresh;
+	gboolean ret;
+
+	g_return_val_if_fail (GPK_IS_AUTO_REFRESH (arefresh), FALSE);
+
+	/* get this each time, as it may have changed behind out back */
+	thresh = gpk_auto_refresh_get_frequency_prefs (arefresh, GPK_CONF_FREQUENCY_GET_UPGRADES);
+	if (thresh == 0) {
+		egg_debug ("not when policy is set to never check for upgrades");
+		return FALSE;
+	}
+
+	/* get the time since the last refresh */
+	ret = pk_control_get_time_since_action (arefresh->priv->control,
+						PK_ROLE_ENUM_GET_DISTRO_UPGRADES, &time_s, NULL);
+	if (!ret) {
+		egg_debug ("failed to get last time");
+		return FALSE;
+	}
+
+	/* have we passed the timout? */
+	if (time_s < thresh) {
+		egg_debug ("not before timeout, thresh=%u, now=%u", thresh, time_s);
+		return FALSE;
+	}
+
+	gpk_auto_refresh_signal_get_upgrades (arefresh);
+	return TRUE;
+}
+
+/**
+ * gpk_auto_refresh_change_state_cb:
+ **/
+static gboolean
+gpk_auto_refresh_change_state_cb (GpkAutoRefresh *arefresh)
+{
+	/* check all actions */
+	gpk_auto_refresh_maybe_refresh_cache (arefresh);
+	gpk_auto_refresh_maybe_get_updates (arefresh);
+	gpk_auto_refresh_maybe_get_upgrades (arefresh);
+	return FALSE;
+}
+
+/**
+ * gpk_auto_refresh_maybe_get_updates_logon_cb:
+ **/
+static gboolean
+gpk_auto_refresh_maybe_get_updates_logon_cb (GpkAutoRefresh *arefresh)
+{
+	gpk_auto_refresh_maybe_get_updates (arefresh);
+	/* never repeat, even if failure */
+	return FALSE;
+}
+
 /**
  * gpk_auto_refresh_change_state:
  **/
 static gboolean
 gpk_auto_refresh_change_state (GpkAutoRefresh *arefresh)
 {
-	guint thresh;
+	gboolean force;
+	guint value;
 
 	g_return_val_if_fail (GPK_IS_AUTO_REFRESH (arefresh), FALSE);
 
-	/* we shouldn't do this early in the session startup */
-	if (arefresh->priv->session_delay == FALSE) {
-		pk_debug ("not when this early in the session");
-		return FALSE;
-	}
-
 	/* no point continuing if we have no network */
-	if (arefresh->priv->network_active == FALSE) {
-		pk_debug ("not when no network");
+	if (!arefresh->priv->network_active) {
+		egg_debug ("not when no network");
 		return FALSE;
 	}
 
-	/* have we been told to never check for updates? */
-	thresh = gpk_auto_refresh_convert_frequency_text (arefresh, GPK_CONF_FREQUENCY_GET_UPDATES);
-	if (thresh == 0) {
-		pk_debug ("not when policy is to never refresh");
-		return FALSE;
+	/* only force a check if the user REALLY, REALLY wants to break
+	 * set policy and have an update at startup */
+	if (!arefresh->priv->force_get_updates_login) {
+		force = gconf_client_get_bool (arefresh->priv->gconf_client, GPK_CONF_FORCE_GET_UPDATES_LOGIN, NULL);
+		if (force) {
+			/* don't immediately send the signal, if we are called during object initialization
+			 * we need to wait until upper layers  finish hooking up to the signal first. */
+			if (arefresh->priv->force_get_updates_login_timeout_id == 0)
+				arefresh->priv->force_get_updates_login_timeout_id =
+					g_timeout_add_seconds (GPK_UPDATES_LOGIN_TIMEOUT, (GSourceFunc) gpk_auto_refresh_maybe_get_updates_logon_cb, arefresh);
+		}
 	}
 
-	/* we do this to get an icon at startup */
-	if (arefresh->priv->sent_get_updates == FALSE) {
-		gpk_auto_refresh_signal_get_updates (arefresh);
-		arefresh->priv->sent_get_updates = TRUE;
-		return TRUE;
-	}
-
-	/* try to do both */
-	gpk_auto_refresh_maybe_refresh_cache (arefresh);
-	gpk_auto_refresh_maybe_get_updates (arefresh);
+	/* wait a little time for things to settle down */
+	if (arefresh->priv->timeout_id != 0)
+		g_source_remove (arefresh->priv->timeout_id);
+	value = gconf_client_get_int (arefresh->priv->gconf_client, GPK_CONF_SESSION_STARTUP_TIMEOUT, NULL);
+	egg_debug ("defering action for %i seconds", value);
+	arefresh->priv->timeout_id = g_timeout_add_seconds (value, (GSourceFunc) gpk_auto_refresh_change_state_cb, arefresh);
 
 	return TRUE;
 }
 
 /**
- * gpk_auto_refresh_idle_cb:
+ * gpk_auto_refresh_gconf_key_changed_cb:
+ *
+ * We might have to do things when the gconf keys change; do them here.
  **/
 static void
-gpk_auto_refresh_idle_cb (DBusGProxy *proxy, gboolean is_idle, GpkAutoRefresh *arefresh)
+gpk_auto_refresh_gconf_key_changed_cb (GConfClient *client, guint cnxn_id, GConfEntry *entry, GpkAutoRefresh *arefresh)
 {
 	g_return_if_fail (GPK_IS_AUTO_REFRESH (arefresh));
-
-	pk_debug ("setting is_idle %i", is_idle);
-	arefresh->priv->session_idle = is_idle;
-	gpk_auto_refresh_change_state (arefresh);
+	if (egg_strequal (entry->key, GPK_CONF_SESSION_STARTUP_TIMEOUT) ||
+	    egg_strequal (entry->key, GPK_CONF_FORCE_GET_UPDATES_LOGIN) ||
+	    egg_strequal (entry->key, GPK_CONF_FREQUENCY_GET_UPDATES) ||
+	    egg_strequal (entry->key, GPK_CONF_FREQUENCY_GET_UPGRADES) ||
+	    egg_strequal (entry->key, GPK_CONF_FREQUENCY_REFRESH_CACHE) ||
+	    egg_strequal (entry->key, GPK_CONF_AUTO_UPDATE) ||
+	    egg_strequal (entry->key, GPK_CONF_UPDATE_BATTERY))
+		gpk_auto_refresh_change_state (arefresh);
 }
 
 /**
- * gpk_auto_refresh_on_battery_cb:
+ * gpk_auto_refresh_session_idle_changed_cb:
  **/
 static void
-gpk_auto_refresh_on_battery_cb (DBusGProxy *proxy, gboolean on_battery, GpkAutoRefresh *arefresh)
+gpk_auto_refresh_session_idle_changed_cb (GpkSession *session, gboolean is_idle, GpkAutoRefresh *arefresh)
 {
 	g_return_if_fail (GPK_IS_AUTO_REFRESH (arefresh));
 
-	pk_debug ("setting on_battery %i", on_battery);
-	arefresh->priv->on_battery = on_battery;
-	gpk_auto_refresh_change_state (arefresh);
+	egg_debug ("setting is_idle %i", is_idle);
+	arefresh->priv->session_idle = is_idle;
+	if (arefresh->priv->session_idle)
+		gpk_auto_refresh_change_state (arefresh);
 }
 
 /**
@@ -357,6 +416,34 @@ gpk_auto_refresh_get_on_battery (GpkAutoRefresh *arefresh)
 	return arefresh->priv->on_battery;
 }
 
+/**
+ * gpk_auto_refresh_convert_network_state:
+ **/
+static gboolean
+gpk_auto_refresh_convert_network_state (GpkAutoRefresh *arefresh, PkNetworkEnum state)
+{
+	/* offline */
+	if (state == PK_NETWORK_ENUM_OFFLINE)
+		return FALSE;
+
+	/* online */
+	if (state == PK_NETWORK_ENUM_ONLINE ||
+	    state == PK_NETWORK_ENUM_WIRED)
+		return TRUE;
+
+	/* check policy */
+	if (state == PK_NETWORK_ENUM_MOBILE)
+		return gconf_client_get_bool (arefresh->priv->gconf_client, GPK_CONF_CONNECTION_USE_MOBILE, NULL);
+
+	/* check policy */
+	if (state == PK_NETWORK_ENUM_WIFI)
+		return gconf_client_get_bool (arefresh->priv->gconf_client, GPK_CONF_CONNECTION_USE_WIFI, NULL);
+
+	/* not recognised */
+	egg_warning ("state unknown: %i", state);
+	return TRUE;
+}
+
 /**
  * gpk_auto_refresh_network_status_changed_cb:
  **/
@@ -365,9 +452,10 @@ gpk_auto_refresh_network_status_changed_cb (PkControl *control, PkNetworkEnum st
 {
 	g_return_if_fail (GPK_IS_AUTO_REFRESH (arefresh));
 
-	arefresh->priv->network_active = pk_enums_contain (state, PK_NETWORK_ENUM_ONLINE);
-	pk_debug ("setting online %i", arefresh->priv->network_active);
-	gpk_auto_refresh_change_state (arefresh);
+	arefresh->priv->network_active = gpk_auto_refresh_convert_network_state (arefresh, state);
+	egg_debug ("setting online %i", arefresh->priv->network_active);
+	if (arefresh->priv->network_active)
+		gpk_auto_refresh_change_state (arefresh);
 }
 
 /**
@@ -381,7 +469,7 @@ gpk_auto_refresh_timeout_cb (gpointer user_data)
 	g_return_val_if_fail (GPK_IS_AUTO_REFRESH (arefresh), FALSE);
 
 	/* debug so we can catch polling */
-	pk_debug ("polling check");
+	egg_debug ("polling check");
 
 	/* triggered once an hour */
 	gpk_auto_refresh_change_state (arefresh);
@@ -391,120 +479,27 @@ gpk_auto_refresh_timeout_cb (gpointer user_data)
 }
 
 /**
- * gpk_auto_refresh_check_delay_cb:
- **/
-static gboolean
-gpk_auto_refresh_check_delay_cb (gpointer user_data)
-{
-	GpkAutoRefresh *arefresh = GPK_AUTO_REFRESH (user_data);
-
-	g_return_val_if_fail (GPK_IS_AUTO_REFRESH (arefresh), FALSE);
-
-	/* debug so we can catch polling */
-	pk_debug ("polling check");
-
-	/* we have waited enough */
-	if (arefresh->priv->session_delay == FALSE) {
-		pk_debug ("setting session delay TRUE");
-		arefresh->priv->session_delay = TRUE;
-	}
-
-	/* if we failed to do the refresh cache at first boot, we'll pick up an event */
-	gpk_auto_refresh_change_state (arefresh);
-
-	/* we don't want to do this timer again as we sent the signal */
-	return FALSE;
-}
-
-/**
- * pk_connection_gpm_changed_cb:
+ * gpk_auto_refresh_client_changed_cb:
  **/
 static void
-pk_connection_gpm_changed_cb (LibGBus *libgbus, gboolean connected, GpkAutoRefresh *arefresh)
+gpk_auto_refresh_client_changed_cb (DkpClient *client, GpkAutoRefresh *arefresh)
 {
-	GError *error = NULL;
 	gboolean on_battery;
-	gboolean ret;
 
 	g_return_if_fail (GPK_IS_AUTO_REFRESH (arefresh));
 
-	pk_debug ("gnome-power-manager connection-changed: %i", connected);
-
-	/* is this valid? */
-	if (connected == FALSE) {
-		if (arefresh->priv->proxy_gpm != NULL) {
-			g_object_unref (arefresh->priv->proxy_gpm);
-			arefresh->priv->proxy_gpm = NULL;
-		}
+	/* get the on-battery state */
+	on_battery = dkp_client_on_battery (arefresh->priv->client);
+	if (on_battery == arefresh->priv->on_battery) {
+		egg_debug ("same state as before, ignoring");
 		return;
 	}
 
-	/* use gnome-power-manager for the battery detection */
-	arefresh->priv->proxy_gpm = dbus_g_proxy_new_for_name_owner (arefresh->priv->connection,
-					  GPM_DBUS_SERVICE, GPM_DBUS_PATH, GPM_DBUS_INTERFACE, &error);
-	if (error != NULL) {
-		pk_warning ("Cannot connect to gnome-power-manager: %s", error->message);
-		g_error_free (error);
-		return;
-	}
-
-	/* setup callbacks and get GetOnBattery if we could connect to g-p-m */
-	dbus_g_proxy_add_signal (arefresh->priv->proxy_gpm, "OnBatteryChanged",
-				 G_TYPE_BOOLEAN, G_TYPE_INVALID);
-	dbus_g_proxy_connect_signal (arefresh->priv->proxy_gpm, "OnBatteryChanged",
-				     G_CALLBACK (gpk_auto_refresh_on_battery_cb),
-				     arefresh, NULL);
-	/* coldplug the battery state */
-	ret = dbus_g_proxy_call (arefresh->priv->proxy_gpm, "GetOnBattery", &error,
-				 G_TYPE_INVALID,
-				 G_TYPE_BOOLEAN, &on_battery,
-				 G_TYPE_INVALID);
-	if (error != NULL) {
-		printf ("DEBUG: ERROR: %s\n", error->message);
-		g_error_free (error);
-	}
-	if (ret) {
-		arefresh->priv->on_battery = on_battery;
-		pk_debug ("setting on battery %i", on_battery);
-	}
-}
-
-/**
- * pk_connection_gs_changed_cb:
- **/
-static void
-pk_connection_gs_changed_cb (LibGBus *libgbus, gboolean connected, GpkAutoRefresh *arefresh)
-{
-	GError *error = NULL;
-
-	g_return_if_fail (GPK_IS_AUTO_REFRESH (arefresh));
-
-	pk_debug ("gnome-screensaver connection-changed: %i", connected);
-
-	/* is this valid? */
-	if (connected == FALSE) {
-		if (arefresh->priv->proxy_gs != NULL) {
-			g_object_unref (arefresh->priv->proxy_gs);
-			arefresh->priv->proxy_gs = NULL;
-		}
-		return;
-	}
-
-	/* use gnome-screensaver for the idle detection */
-	arefresh->priv->proxy_gs = dbus_g_proxy_new_for_name_owner (arefresh->priv->connection,
-					  GS_DBUS_SERVICE, GS_DBUS_PATH, GS_DBUS_INTERFACE, &error);
-	if (error != NULL) {
-		pk_warning ("Cannot connect to gnome-screensaver: %s", error->message);
-		g_error_free (error);
-		return;
-	}
-	/* get SessionIdleChanged */
-	dbus_g_proxy_add_signal (arefresh->priv->proxy_gs, "SessionIdleChanged",
-				 G_TYPE_BOOLEAN, G_TYPE_INVALID);
-	dbus_g_proxy_connect_signal (arefresh->priv->proxy_gs, "SessionIdleChanged",
-				     G_CALLBACK (gpk_auto_refresh_idle_cb),
-				     arefresh, NULL);
-
+	/* save in local cache */
+	egg_debug ("setting on_battery %i", on_battery);
+	arefresh->priv->on_battery = on_battery;
+	if (!on_battery)
+		gpk_auto_refresh_change_state (arefresh);
 }
 
 /**
@@ -514,58 +509,52 @@ pk_connection_gs_changed_cb (LibGBus *libgbus, gboolean connected, GpkAutoRefres
 static void
 gpk_auto_refresh_init (GpkAutoRefresh *arefresh)
 {
-	guint value;
-	GError *error = NULL;
 	PkNetworkEnum state;
 
 	arefresh->priv = GPK_AUTO_REFRESH_GET_PRIVATE (arefresh);
 	arefresh->priv->on_battery = FALSE;
-	arefresh->priv->session_idle = FALSE;
 	arefresh->priv->network_active = FALSE;
-	arefresh->priv->session_delay = FALSE;
-	arefresh->priv->sent_get_updates = FALSE;
-
-	arefresh->priv->proxy_gs = NULL;
-	arefresh->priv->proxy_gpm = NULL;
+	arefresh->priv->force_get_updates_login = FALSE;
+	arefresh->priv->timeout_id = 0;
+	arefresh->priv->force_get_updates_login_timeout_id = 0;
 
 	/* we need to know the updates frequency */
 	arefresh->priv->gconf_client = gconf_client_get_default ();
 
+	/* watch gnome-packagekit keys */
+	gconf_client_add_dir (arefresh->priv->gconf_client, GPK_CONF_DIR,
+			      GCONF_CLIENT_PRELOAD_NONE, NULL);
+	gconf_client_notify_add (arefresh->priv->gconf_client, GPK_CONF_DIR,
+				 (GConfClientNotifyFunc) gpk_auto_refresh_gconf_key_changed_cb,
+				 arefresh, NULL, NULL);
+
 	/* we need to query the last cache refresh time */
 	arefresh->priv->control = pk_control_new ();
 	g_signal_connect (arefresh->priv->control, "network-state-changed",
 			  G_CALLBACK (gpk_auto_refresh_network_status_changed_cb), arefresh);
-	state = pk_control_get_network_state (arefresh->priv->control);
-	if (pk_enums_contain (state, PK_NETWORK_ENUM_ONLINE)) {
-		arefresh->priv->network_active = TRUE;
-	}
+	state = pk_control_get_network_state (arefresh->priv->control, NULL);
+	arefresh->priv->network_active = gpk_auto_refresh_convert_network_state (arefresh, state);
 
-	/* connect to session bus */
-	arefresh->priv->connection = dbus_g_bus_get (DBUS_BUS_SESSION, &error);
-	if (error != NULL) {
-		pk_warning ("Cannot connect to session bus: %s", error->message);
-		g_error_free (error);
-		return;
-	}
+	/* use a DkpClient */
+	arefresh->priv->client = dkp_client_new ();
+	g_signal_connect (arefresh->priv->client, "changed",
+			  G_CALLBACK (gpk_auto_refresh_client_changed_cb), arefresh);
 
-	/* watch gnome-screensaver */
-	arefresh->priv->gbus_gs = libgbus_new ();
-	g_signal_connect (arefresh->priv->gbus_gs, "connection-changed",
-			  G_CALLBACK (pk_connection_gs_changed_cb), arefresh);
-	libgbus_assign (arefresh->priv->gbus_gs, LIBGBUS_SESSION, GS_DBUS_SERVICE);
+	/* get the battery state */
+	arefresh->priv->on_battery = dkp_client_on_battery (arefresh->priv->client);
+	egg_debug ("setting on battery %i", arefresh->priv->on_battery);
 
-	/* watch gnome-power-manager */
-	arefresh->priv->gbus_gpm = libgbus_new ();
-	g_signal_connect (arefresh->priv->gbus_gpm, "connection-changed",
-			  G_CALLBACK (pk_connection_gpm_changed_cb), arefresh);
-	libgbus_assign (arefresh->priv->gbus_gpm, LIBGBUS_SESSION, GPM_DBUS_SERVICE);
+	/* use gnome-session for the idle detection */
+	arefresh->priv->session = gpk_session_new ();
+	g_signal_connect (arefresh->priv->session, "idle_changed",
+			  G_CALLBACK (gpk_auto_refresh_session_idle_changed_cb), arefresh);
+	arefresh->priv->session_idle = gpk_session_get_idle (arefresh->priv->session);
 
 	/* we check this in case we miss one of the async signals */
 	g_timeout_add_seconds (GPK_AUTO_REFRESH_PERIODIC_CHECK, gpk_auto_refresh_timeout_cb, arefresh);
 
-	/* wait a little bit for login to quiece, even if everything is okay */
-	value = gconf_client_get_int (arefresh->priv->gconf_client, GPK_CONF_SESSION_STARTUP_TIMEOUT, NULL);
-	g_timeout_add_seconds (value, gpk_auto_refresh_check_delay_cb, arefresh);
+	/* check system state */
+	gpk_auto_refresh_change_state (arefresh);
 }
 
 /**
@@ -582,18 +571,15 @@ gpk_auto_refresh_finalize (GObject *object)
 	arefresh = GPK_AUTO_REFRESH (object);
 	g_return_if_fail (arefresh->priv != NULL);
 
-	g_object_unref (arefresh->priv->control);
-	g_object_unref (arefresh->priv->gbus_gs);
-	g_object_unref (arefresh->priv->gbus_gpm);
-	g_object_unref (arefresh->priv->gconf_client);
+	if (arefresh->priv->timeout_id != 0)
+		g_source_remove (arefresh->priv->timeout_id);
+	if (arefresh->priv->force_get_updates_login_timeout_id != 0)
+		g_source_remove (arefresh->priv->force_get_updates_login_timeout_id);
 
-	/* only unref the proxies if they were ever set */
-	if (arefresh->priv->proxy_gs != NULL) {
-		g_object_unref (arefresh->priv->proxy_gs);
-	}
-	if (arefresh->priv->proxy_gpm != NULL) {
-		g_object_unref (arefresh->priv->proxy_gpm);
-	}
+	g_object_unref (arefresh->priv->control);
+	g_object_unref (arefresh->priv->gconf_client);
+	g_object_unref (arefresh->priv->client);
+	g_object_unref (arefresh->priv->session);
 
 	G_OBJECT_CLASS (gpk_auto_refresh_parent_class)->finalize (object);
 }
