@@ -177,7 +177,6 @@ gpk_check_update_about_dialog_url_cb (GtkAboutDialog *about, const char *address
 {
 	GError *error = NULL;
 	gboolean ret;
-	char *cmdline;
 	GdkScreen *gscreen;
 	GtkWidget *error_dialog;
 	gchar *url;
@@ -190,19 +189,7 @@ gpk_check_update_about_dialog_url_cb (GtkAboutDialog *about, const char *address
 
 	gscreen = gtk_window_get_screen (GTK_WINDOW (about));
 
-	cmdline = g_strconcat ("xdg-open ", url, NULL);
-	ret = gdk_spawn_command_line_on_screen (gscreen, cmdline, &error);
-	g_free (cmdline);
-
-	if (ret)
-		goto out;
-
-	g_error_free (error);
-	error = NULL;
-
-	cmdline = g_strconcat ("gnome-open ", url, NULL);
-	ret = gdk_spawn_command_line_on_screen (gscreen, cmdline, &error);
-	g_free (cmdline);
+	ret = gtk_show_uri (gscreen, url, gtk_get_current_event_time (), &error);
 
 	if (!ret) {
 		error_dialog = gtk_message_dialog_new (GTK_WINDOW (about),
@@ -218,7 +205,6 @@ gpk_check_update_about_dialog_url_cb (GtkAboutDialog *about, const char *address
 		g_error_free (error);
 	}
 
-out:
 	g_free (url);
 }
 
@@ -363,7 +349,11 @@ gpk_check_update_update_system (GpkCheckUpdate *cupdate)
 		goto out;
 	}
 
+#if PK_CHECK_VERSION(0,5,0)
+	ret = pk_client_update_system (cupdate->priv->client_primary, TRUE, &error);
+#else
 	ret = pk_client_update_system (cupdate->priv->client_primary, &error);
+#endif
 	if (!ret) {
 		/* we failed, show the icon */
 		egg_warning ("cannot update system: %s", error->message);
@@ -488,7 +478,11 @@ gpk_check_update_libnotify_cb (NotifyNotification *notification, gchar *action,
 
 		/* just update the important updates */
 		package_ids = pk_package_ids_from_array (cupdate->priv->important_updates_array);
+#if PK_CHECK_VERSION(0,5,0)
+		ret = pk_client_update_packages (cupdate->priv->client_primary, TRUE, package_ids, &error);
+#else
 		ret = pk_client_update_packages (cupdate->priv->client_primary, package_ids, &error);
+#endif
 		if (!ret) {
 			egg_warning ("Individual updates failed: %s", error->message);
 			g_error_free (error);
@@ -571,6 +565,7 @@ gpk_check_update_critical_updates_warning (GpkCheckUpdate *cupdate, const gchar
 	}
 
 	/* do the bubble */
+	egg_debug ("title=%s, message=%s", title, message);
 	notification = notify_notification_new (title, message, "help-browser", NULL);
 	if (notification == NULL) {
 		egg_warning ("failed to get bubble");
@@ -928,6 +923,8 @@ gpk_check_update_process_updates (GpkCheckUpdate *cupdate, PkPackageList *list,
 	if (update == GPK_UPDATE_ENUM_NONE) {
 		egg_debug ("not updating as policy NONE");
 
+		/* TODO: use ca_gtk_context_get_for_screen to allow use of GDK_MULTIHEAD_SAFE */
+
 		/* play the sound, using sounds from the naming spec */
 		ca_context_play (ca_gtk_context_get (), 0,
 				 /* TODO: add a new sound to the spec */
@@ -979,7 +976,11 @@ gpk_check_update_process_updates (GpkCheckUpdate *cupdate, PkPackageList *list,
 
 		/* convert */
 		package_ids = pk_package_ids_from_array (security_array);
+#if PK_CHECK_VERSION(0,5,0)
+		ret = pk_client_update_packages (cupdate->priv->client_primary, TRUE, package_ids, &error);
+#else
 		ret = pk_client_update_packages (cupdate->priv->client_primary, package_ids, &error);
+#endif
 		if (!ret) {
 			egg_warning ("Individual updates failed: %s", error->message);
 			g_error_free (error);
@@ -1013,7 +1014,7 @@ out:
 static gboolean
 gpk_check_update_query_updates_idle_cb (GpkCheckUpdate *cupdate)
 {
-	egg_warning ("idle cb");
+	egg_debug ("idle cb");
 	gpk_check_update_query_updates (cupdate, TRUE);
 	return FALSE;
 }
@@ -1024,10 +1025,19 @@ gpk_check_update_query_updates_idle_cb (GpkCheckUpdate *cupdate)
 static void
 gpk_check_update_updates_changed_cb (PkControl *control, GpkCheckUpdate *cupdate)
 {
+	guint thresh;
+
 	g_return_if_fail (GPK_IS_CHECK_UPDATE (cupdate));
 
+	/* if we don't want to auto check for updates, don't do this either */
+	thresh = gconf_client_get_int (cupdate->priv->gconf_client, GPK_CONF_FREQUENCY_GET_UPDATES, NULL);
+	if (thresh == 0) {
+		egg_debug ("not when policy is to never get updates");
+		return;
+	}
+
 	/* now try to get newest update list */
-	egg_warning ("updates changed");
+	egg_debug ("updates changed, so getting new update list");
 	g_idle_add ((GSourceFunc) gpk_check_update_query_updates_idle_cb, cupdate);
 }
 
@@ -1237,7 +1247,7 @@ gpk_check_update_error_code_cb (PkClient *client, PkErrorCodeEnum code, const gc
 	}
 
 	/* ignore the ones we can handle */
-	if (code == PK_ERROR_ENUM_GPG_FAILURE) {
+	if (pk_error_code_is_need_untrusted (code)) {
 		egg_debug ("error ignored as we're handling %s\n%s", pk_error_enum_to_text (code), details);
 		return;
 	}
@@ -1426,6 +1436,15 @@ gpk_check_update_finished_cb (PkClient *client, PkExitEnum exit_enum, guint runt
 	pk_client_get_role (client, &role, NULL, NULL);
 	egg_debug ("role: %s, exit: %s", pk_role_enum_to_text (role), pk_exit_enum_to_text (exit_enum));
 
+	/* need to handle retry with only_trusted=FALSE */
+	if (client == cupdate->priv->client_primary &&
+	    exit_enum == PK_EXIT_ENUM_NEED_UNTRUSTED) {
+		egg_debug ("need to handle untrusted");
+		pk_client_set_only_trusted (client, FALSE);
+		gpk_check_update_primary_requeue (cupdate);
+		return;
+	}
+
 	/* we've just agreed to auth */
 	if (role == PK_ROLE_ENUM_INSTALL_SIGNATURE) {
 		if (exit_enum == PK_EXIT_ENUM_SUCCESS)
@@ -1443,7 +1462,11 @@ gpk_check_update_finished_cb (PkClient *client, PkExitEnum exit_enum, guint runt
 	/* get-upgrades */
 	if (role == PK_ROLE_ENUM_GET_DISTRO_UPGRADES &&
 	    exit_enum == PK_EXIT_ENUM_SUCCESS) {
-		array = pk_client_get_cached_objects (client);
+#if PK_CHECK_VERSION(0,5,0)
+		array = pk_client_get_distro_upgrade_list (client);
+#else
+		array = pk_client_get_cached_objects (client); /* removed in 0.5.x */
+#endif
 		gpk_check_update_process_distro_upgrades (cupdate, array);
 		g_object_unref (array);
 	}
