@@ -25,12 +25,13 @@
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
 #include <locale.h>
+#include <dbus/dbus-glib.h>
 
 #include "egg-debug.h"
 
 #include "gpk-common.h"
-#include "gpk-client.h"
 #include "gpk-error.h"
+#include "gpk-dbus.h"
 
 /**
  * main:
@@ -41,12 +42,17 @@ main (int argc, char *argv[])
 	GOptionContext *context;
 	gboolean ret;
 	gboolean verbose = FALSE;
-	GError *error;
-	GpkClient *gclient;
+	GError *error = NULL;
+	DBusGConnection *connection;
+	DBusGProxy *proxy = NULL;
+	gchar **files = NULL;
 
 	const GOptionEntry options[] = {
 		{ "verbose", 'v', 0, G_OPTION_ARG_NONE, &verbose,
 		  _("Show extra debugging information"), NULL },
+		{ G_OPTION_REMAINING, '\0', 0, G_OPTION_ARG_FILENAME_ARRAY, &files,
+		/* TRANSLATORS: command line option: a list of files to install */
+		  _("Local files to install"), NULL },
 		{ NULL}
 	};
 
@@ -74,28 +80,57 @@ main (int argc, char *argv[])
 	/* TRANSLATORS: application name to pass to to the user if there are not enough privs */
 	ret = gpk_check_privileged_user (_("Single File Installer"), TRUE);
 	if (!ret)
-		return 1;
+		goto out;
 
-	if (argc < 2) {
+	if (files == NULL) {
 		/* TRANSLATORS: nothing done */
 		gpk_error_dialog (_("Failed to install a package to provide a file"),
 				  /* TRANSLATORS: nothig was specified */
 				  _("You need to specify a filename to install"), NULL);
-		return 1;
-	}
-	if (argc > 2) {
-		/* TRANSLATORS: nothing done */
-		gpk_error_dialog (_("Failed to install packages to provide files"),
-				  /* TRANSLATORS: more than one thing was specified */
-				  _("You can only specify one filename to install"), NULL);
-		return 1;
+		goto out;
 	}
 
-	error = NULL;
-	gclient = gpk_client_new ();
-	gpk_client_set_interaction (gclient, GPK_CLIENT_INTERACT_ALWAYS);
-	ret = gpk_client_install_provide_file (gclient, argv[1], NULL);
-	g_object_unref (gclient);
+	/* check dbus connections, exit if not valid */
+	connection = dbus_g_bus_get (DBUS_BUS_SESSION, &error);
+	if (connection == NULL) {
+		egg_warning ("%s", error->message);
+		goto out;
+	}
 
+	/* get a connection */
+	proxy = dbus_g_proxy_new_for_name (connection,
+					   "org.freedesktop.PackageKit",
+					   "/org/freedesktop/PackageKit",
+					   "org.freedesktop.PackageKit.Modify");
+	if (proxy == NULL) {
+		egg_warning ("Cannot connect to session service");
+		goto out;
+	}
+
+	/* don't timeout, as dbus-glib sets the timeout ~25 seconds */
+	dbus_g_proxy_set_default_timeout (proxy, INT_MAX);
+
+	/* do method */
+	ret = dbus_g_proxy_call (proxy, "InstallProvideFiles", &error,
+				 G_TYPE_UINT, 0, /* xid */
+				 G_TYPE_STRV, files, /* data */
+				 G_TYPE_STRING, "", /* interaction */
+				 G_TYPE_INVALID,
+				 G_TYPE_INVALID);
+	if (!ret && !gpk_ignore_session_error (error)) {
+		/* TRANSLATORS: This is when the specified DBus method did not execute successfully */
+		gpk_error_dialog (_("The action could not be completed"),
+				  /* TRANSLATORS: we don't have anything more useful to translate. sorry. */
+				  _("The request failed. More details are available in the detailed report."),
+				  error->message);
+		egg_warning ("%s", error->message);
+		goto out;
+	}
+out:
+	if (error != NULL)
+		g_error_free (error);
+	if (proxy != NULL)
+		g_object_unref (proxy);
+	g_strfreev (files);
 	return !ret;
 }
